@@ -37,7 +37,7 @@
        Keith Owens <kaos@ocs.com.au> April 2000.
    */
 
-#ident "$Id: depmod.c 1.25 Tue, 10 Oct 2000 12:16:02 +1100 kaos $"
+#ident "$Id: depmod.c 1.26 Sat, 21 Oct 2000 15:17:56 +1100 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -101,6 +101,57 @@ struct isapnp_card_id {
 	} devs[ISAPNP_CARD_DEVS];	/* logical devices */
 };
 
+/* Extracted from 2.4.0-test10-pre3 + David Brownell usb-device_id patch 2 */
+#ifndef __u16
+#define __u16 u_int16_t
+#endif
+#ifndef __u8
+#define __u8 u_int8_t
+#endif
+/*
+ * Device table entry for "new style" table-driven USB drivers.
+ * User mode code can read these tables to choose which modules to load.
+ * Declare the table as __devinitdata, and as a MODULE_DEVICE_TABLE.
+ *
+ * With a device table provide bind() instead of probe().  Then the
+ * third bind() parameter will point to a matching entry from this
+ * table.  (Null value reserved.)
+ * 
+ * Terminate the driver's table with an all-zeroes entry.
+ * Init the fields you care about; zeroes are not used in comparisons.
+ */
+struct usb_device_id {
+	/*
+	 * vendor/product codes are checked, if vendor is nonzero
+	 * Range is for device revision (bcdDevice), inclusive;
+	 * zero values here mean range isn't considered
+	 */
+	__u16		idVendor;
+	__u16		idProduct;
+	__u16		bcdDevice_lo, bcdDevice_hi;
+
+	/*
+	 * if device class != 0, these can be match criteria;
+	 * but only if this bDeviceClass value is nonzero
+	 */
+	__u8		bDeviceClass;
+	__u8		bDeviceSubClass;
+	__u8		bDeviceProtocol;
+
+	/*
+	 * if interface class != 0, these can be match criteria;
+	 * but only if this bInterfaceClass value is nonzero
+	 */
+	__u8		bInterfaceClass;
+	__u8		bInterfaceSubClass;
+	__u8		bInterfaceProtocol;
+
+	/*
+	 * for driver's use; not involved in driver matching.
+	 */
+	unsigned long	driver_info;
+};
+
 typedef struct MODULE {
 	char *name;
 	int resolved;
@@ -114,6 +165,8 @@ typedef struct MODULE {
 	int n_isapnp_device;
 	struct isapnp_card_id *isapnp_card;
 	int n_isapnp_card;
+	struct usb_device_id *usb_device;
+	int n_usb_device;
 } MODULE;
 
 typedef enum SYM_STATUS {
@@ -400,6 +453,37 @@ static void extract_isapnp_card_id(struct obj_file *f, void *image, unsigned lon
 	}
 }
 
+/*
+ *	Extract any usb_device_table from the module.
+ *	Note: assumes same machine and arch for depmod and module.
+ */
+static void extract_usb_device_id(struct obj_file *f, void *image, unsigned long m_size, MODULE *mod)
+{
+	struct usb_device_id usb_device;
+	ElfW(Addr) ref_usb, ref_ref_usb;
+	unsigned long usb_device_size;
+	ref_usb = obj_symbol_final_value(f, obj_find_symbol(f, "__module_usb_device_size"));
+	if (!in_range(f, m_size, ref_usb, sizeof(usb_device_size)))
+		return;
+	memcpy(&usb_device_size, (char *)image + ref_usb - f->baseaddr, sizeof(usb_device_size));
+	if (usb_device_size != sizeof(usb_device)) {
+		error("Unexpected value (%ld) for usb_device_size%s", usb_device_size, has_kernel_changed);
+		exit(-1);
+	}
+	ref_ref_usb = obj_symbol_final_value(f, obj_find_symbol(f, "__module_usb_device_table"));
+	if (!in_range(f, m_size, ref_ref_usb, sizeof(ref_usb)))
+		return;
+	memcpy(&ref_usb, (char *)image + ref_ref_usb - f->baseaddr, sizeof(ref_usb));
+	while (in_range(f, m_size, ref_usb, sizeof(usb_device))) {
+		memcpy(&usb_device, (char *)image + ref_usb - f->baseaddr, sizeof(usb_device));
+		ref_usb += sizeof(usb_device);
+		if (!usb_device.idVendor)
+			break;
+		mod->usb_device = xrealloc(mod->usb_device, ++(mod->n_usb_device)*sizeof(*(mod->usb_device)));
+		mod->usb_device[mod->n_usb_device-1] = usb_device;
+	}
+}
+
 /*
  *	Read the symbols in an object and register them in the symbol table.
  *	Return -1 if there is an error.
@@ -575,6 +659,7 @@ static int loadobj(const char *objname, int ignore_suffix)
 	extract_pci_device_id(f, image, m_size, mod);
 	extract_isapnp_device_id(f, image, m_size, mod);
 	extract_isapnp_card_id(f, image, m_size, mod);
+	extract_usb_device_id(f, image, m_size, mod);
 	free(image);
 
 	obj_free(f);
@@ -705,15 +790,18 @@ static int addksyms(char *file_syms)
  *	Print the dependancies in the depfile (or stdout if depfile is NULL).
  *	Print the pcimap in the pcimapfile (or stdout if pcimapfile is NULL).
  *	Print the isapnpmap in the isapnpmapfile (or stdout if isapnpmapfile is NULL).
+ *	Print the usbmap in the usbmapfile (or stdout if usbmapfile is NULL).
  */
 static void prtdepend(char *base_dir,
 		      const char *pdepfile,
 		      const char *ppcimapfile,
-		      const char *pisapnpmapfile)
+		      const char *pisapnpmapfile,
+		      const char *pusbmapfile)
 {
 	FILE *dep = stdout;
 	FILE *pcimap = stdout;
 	FILE *isapnpmap = stdout;
+	FILE *usbmap = stdout;
 	MODULE **tbdep;
 	MODULE *ptmod;
 	int i;
@@ -740,6 +828,13 @@ static void prtdepend(char *base_dir,
 		}
 	}
 
+	if (pusbmapfile != NULL) {
+		if ((usbmap = fopen(pusbmapfile, "w")) == NULL) {
+			error("Can't open %s for writing", pusbmapfile);
+			exit(-1);
+		}
+	}
+
 	skipchars = strlen(base_dir);
 	tbdep = (MODULE **)alloca(sizeof(MODULE) * n_modules);
 
@@ -889,11 +984,62 @@ static void prtdepend(char *base_dir,
 		free(name);
 	}
 
+	ptmod = modules;
+	fprintf(usbmap, "# module             idVendor idProduct bcdDevice_lo bcdDevice_hi"
+		" bDeviceClass bDeviceSubClass bDeviceProtocol bInterfaceClass bInterfaceSubClass"
+		" bInterfaceProtocol driver_info\n");
+	for (i = 0; i < n_modules; i++, ptmod++) {
+		int j, l;
+		struct usb_device_id *usb_device = ptmod->usb_device;
+		char *name, *shortname;
+		if (!ptmod->n_usb_device)
+			continue;
+		name = xstrdup(ptmod->name);
+		shortname = strrchr(name, '/');
+		if (!shortname)
+			shortname = name;	/* should never happen */
+		else {
+			++shortname;
+			l = strlen(shortname);
+			if (l > 3 && strcmp(shortname+l-3, ".gz") == 0) {
+				*(shortname+l-3) = '\0';
+				l -= 3;
+			}
+			if (l > 2 && strcmp(shortname+l-2, ".o") == 0) {
+				*(shortname+l-2) = '\0';
+				l -= 2;
+			} else if (l > 4 && strcmp(shortname+l-4, ".mod") == 0) {
+				*(shortname+l-4) = '\0';
+				l -= 4;
+			}
+		}
+		for (j = 0; j < ptmod->n_usb_device; j++, usb_device++) {
+			fprintf(usbmap, "%-20s 0x%0*x   0x%0*x    0x%0*x       0x%0*x       "
+				"0x%0*x         0x%0*x            0x%0*x            0x%0*x            0x%0*x            "
+				"0x%0*x            0x%0*lx\n",
+				shortname,
+				2*sizeof(usb_device->idVendor), usb_device->idVendor,
+				2*sizeof(usb_device->idProduct), usb_device->idProduct,
+				2*sizeof(usb_device->bcdDevice_lo), usb_device->bcdDevice_lo,
+				2*sizeof(usb_device->bcdDevice_hi), usb_device->bcdDevice_hi,
+				2*sizeof(usb_device->bDeviceClass), usb_device->bDeviceClass,
+				2*sizeof(usb_device->bDeviceSubClass), usb_device->bDeviceSubClass,
+				2*sizeof(usb_device->bDeviceProtocol), usb_device->bDeviceProtocol,
+				2*sizeof(usb_device->bInterfaceClass), usb_device->bInterfaceClass,
+				2*sizeof(usb_device->bInterfaceSubClass), usb_device->bInterfaceSubClass,
+				2*sizeof(usb_device->bInterfaceProtocol), usb_device->bInterfaceProtocol,
+				2*sizeof(usb_device->driver_info), usb_device->driver_info);
+		}
+		free(name);
+	}
+
 	/* Close depfile last, critical timestamp */
 	if (ppcimapfile != NULL)
 		fclose(pcimap);
 	if (pisapnpmapfile != NULL)
 		fclose(isapnpmap);
+	if (pusbmapfile != NULL)
+		fclose(usbmap);
 	if (pdepfile != NULL)
 		fclose(dep);
 }
@@ -1087,7 +1233,8 @@ int DEPMOD_MAIN(int argc, char **argv)
 			prtdepend(base_dir,
 				  nflag ? NULL : depfile,
 				  nflag ? NULL : pcimapfile,
-				  nflag ? NULL : isapnpmapfile);
+				  nflag ? NULL : isapnpmapfile,
+				  nflag ? NULL : usbmapfile);
 		}
 	} else {
 		/* not stdmode */
@@ -1095,7 +1242,7 @@ int DEPMOD_MAIN(int argc, char **argv)
 			for (; argc > 0 && ret != -1; ++argv, --argc)
 				loadobj(*argv, ignore_suffix);
 			resolve();
-			prtdepend(base_dir, NULL, NULL, NULL);
+			prtdepend(base_dir, NULL, NULL, NULL, NULL);
 		}
 	}
 
