@@ -48,9 +48,11 @@
        Keith Owens <kaos@ocs.com.au> April 2000.
      archdata support
        Keith Owens <kaos@ocs.com.au> August 2000.
+     Add insmod -O, move print map before sys_init_module.
+       Keith Owens <kaos@ocs.com.au> October 2000.
    */
 
-#ident "$Id: insmod.c 1.27 Sun, 10 Sep 2000 11:25:29 +1100 kaos $"
+#ident "$Id: insmod.c 1.29 Sat, 21 Oct 2000 15:31:41 +1100 kaos $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -1021,12 +1023,13 @@ static int add_archdata(struct obj_file *f,
 
 
 static int init_module(const char *m_name, struct obj_file *f,
-		       unsigned long m_size)
+		       unsigned long m_size, const char *blob_name,
+		       unsigned int noload, unsigned int flag_load_map)
 {
 	struct module *module;
 	struct obj_section *sec;
 	void *image;
-	int ret;
+	int ret = 0;
 	tgt_long m_addr;
 
 	sec = obj_find_section(f, ".this");
@@ -1086,11 +1089,33 @@ static int init_module(const char *m_name, struct obj_file *f,
 	image = xmalloc(m_size);
 	obj_create_image(f, image);
 
-	ret = sys_init_module(m_name, (struct module *) image);
-	if (ret) {
-		error("init_module: %m");
-		lprintf("Hint: insmod errors can be caused by incorrect module parameters, "
-			"including invalid IO or IRQ parameters");
+	if (flag_load_map)
+		print_load_map(f);
+
+	if (blob_name) {
+		int fd, l;
+		fd = open(blob_name, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
+		if (fd < 0) {
+			error("open %s failed %m", blob_name);
+			ret = -1;
+		}
+		else {
+			if ((l = write(fd, image, m_size)) != m_size) {
+				error("write %s failed %m", blob_name);
+				ret = -1;
+			}
+			close(fd);
+		}
+	}
+
+	if (ret == 0 && !noload) {
+		fflush(stdout);		/* Flush any debugging output */
+		ret = sys_init_module(m_name, (struct module *) image);
+		if (ret) {
+			error("init_module: %m");
+			lprintf("Hint: insmod errors can be caused by incorrect module parameters, "
+				"including invalid IO or IRQ parameters");
+		}
 	}
 
 	free(image);
@@ -1223,6 +1248,7 @@ void insmod_usage(void)
 	      "  -m, --map             Generate load map (so crashes can be traced)\n"
 	      "  -n, --noload          Don't load, just show\n"
 	      "  -o NAME, --name=NAME  Set internal module name to NAME\n"
+	      "  -O NAME, --blob=NAME  Save the object as a binary blob in NAME\n"
 	      "  -p, --poll            Poll mode; check if the module matches the kernel\n"
 	      "  -q, --quiet           Don't print unresolved symbols\n"
 	      "  -s, --syslog          Report errors via syslog\n"
@@ -1260,6 +1286,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		{"map", 0, 0, 'm'},
 		{"noload", 0, 0, 'n'},
 		{"name", 1, 0, 'o'},
+		{"blob", 1, 0, 'O'},
 		{"poll", 0, 0, 'p'},
 		{"syslog", 0, 0, 's'},
 		{"verbose", 0, 0, 'v'},
@@ -1276,6 +1303,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		{0, 0, 0, 0}
 	};
 	char *m_name = NULL;
+	char *blob_name = NULL;		/* Save object as binary blob */
 	int m_version;
 	ElfW(Addr) m_addr;
 	unsigned long m_size;
@@ -1298,7 +1326,7 @@ int INSMOD_MAIN(int argc, char **argv)
 	errors = optind = 0;
 
 	/* Process the command line.  */
-	while ((o = getopt_long(argc, argv, "fhkmno:pqsvVxXLP:yYr",
+	while ((o = getopt_long(argc, argv, "fhkmno:O:pqsvVxXLP:yYr",
 				&long_opts[0], NULL)) != EOF)
 		switch (o) {
 		case 'f':	/* force loading */
@@ -1319,6 +1347,9 @@ int INSMOD_MAIN(int argc, char **argv)
 		case 'o':	/* name the output module */
 			m_name = optarg;
 			break;
+		case 'O':	/* save the output module object */
+			blob_name = optarg;
+			break;
 		case 'p':	/* silent poll mode */
 			flag_silent_poll = 1;
 			break;
@@ -1563,23 +1594,19 @@ int INSMOD_MAIN(int argc, char **argv)
 	if (add_kallsyms(f, &kallsyms))
 		goto out;
 
-	if (!noload) {
 #ifdef COMPAT_2_0
-		if (k_new_syscalls)
-			init_module(m_name, f, m_size);
-		else
-			old_init_module(m_name, f, m_size);
+	if (k_new_syscalls)
+		init_module(m_name, f, m_size, blob_name, noload, flag_load_map);
+	else if (!noload)
+		old_init_module(m_name, f, m_size);
 #else
-		init_module(m_name, f, m_size);
+	init_module(m_name, f, m_size, blob_name, noload, flag_load_map);
 #endif
-	}
 	if (errors) {
 		if (!noload)
 			delete_module(m_name);
 		goto out;
 	}
-	if (flag_load_map)
-		print_load_map(f);
 	exit_status = 0;
 
       out:
