@@ -21,7 +21,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_load.c 1.9 Tue, 10 Oct 2000 22:05:46 +1100 kaos $"
+#ident "$Id: obj_load.c 1.10 Sat, 21 Oct 2000 16:41:21 +1100 kaos $"
 
 #include <alloca.h>
 #include <string.h>
@@ -189,6 +189,12 @@ obj_load (int fp, Elf32_Half e_type, const char *filename)
     {
       struct obj_section *sec = f->sections[i];
 
+      /* .modinfo should be contents only but gcc has no attribute for that.
+       * The kernel may have marked .modinfo as ALLOC, ignore this bit.
+       */
+      if (strcmp(sec->name, ".modinfo") == 0)
+        sec->header.sh_flags &= ~SHF_ALLOC;
+
       if (sec->header.sh_flags & SHF_ALLOC)
 	obj_insert_section_load_order(f, sec);
 
