@@ -102,6 +102,7 @@ char *insmod_opt = NULL;
 char *depfile = NULL;
 char *pcimapfile = NULL;
 char *isapnpmapfile = NULL;
+char *usbmapfile = NULL;
 char *config_file = NULL;	/* Which file was actually used */
 time_t config_mtime;
 int root_check_off = CONFIG_ROOT_CHECK_OFF;	/* Default is modules must be owned by root */
@@ -129,6 +130,7 @@ void verbose(const char *ctl,...)
 static time_t dep_mtime;
 static time_t pcimap_mtime;
 static time_t isapnpmap_mtime;
+static time_t usbmap_mtime;
 static int check_update (const char *file, const struct stat *sb)
 {
 	int len = strlen(file);
@@ -137,7 +139,8 @@ static int check_update (const char *file, const struct stat *sb)
 		return 0;
 	if (sb->st_mtime <= dep_mtime &&
 	    sb->st_mtime <= pcimap_mtime &&
-	    sb->st_mtime <= isapnpmap_mtime)
+	    sb->st_mtime <= isapnpmap_mtime &&
+	    sb->st_mtime <= usbmap_mtime)
 		return 0;
 
 	if (len > 2 && !strcmp(file + len - 2, ".o"))
@@ -153,7 +156,7 @@ static int check_update (const char *file, const struct stat *sb)
 
 static int need_update (const char *force_ver, const char *base_dir)
 {
-	struct stat statdep, statpcimap, statisapnpmap, tmp;
+	struct stat statdep, statpcimap, statisapnpmap, statusbmap, tmp;
 	char dep[PATH_MAX];
 	uname (&uts_info);
 	if (!force_ver)
@@ -165,7 +168,8 @@ static int need_update (const char *force_ver, const char *base_dir)
 
 	if (stat (depfile, &statdep) ||
 	    stat (pcimapfile, &statpcimap) ||
-	    stat (isapnpmapfile, &statisapnpmap))
+	    stat (isapnpmapfile, &statisapnpmap) ||
+	    stat (usbmapfile, &statusbmap))
 		/* No dependency file yet, so we need to build it. */
 		return 1;
 
@@ -175,13 +179,15 @@ static int need_update (const char *force_ver, const char *base_dir)
 
 	if (tmp.st_mtime > statdep.st_mtime ||
 	    tmp.st_mtime > statpcimap.st_mtime ||
-	    tmp.st_mtime > statisapnpmap.st_mtime)
+	    tmp.st_mtime > statisapnpmap.st_mtime ||
+	    tmp.st_mtime > statusbmap.st_mtime)
 		/* Config file is newer. */
 		return 1;
 
 	dep_mtime = statdep.st_mtime;
 	pcimap_mtime = statpcimap.st_mtime;
 	isapnpmap_mtime = statisapnpmap.st_mtime;
+	usbmap_mtime = statusbmap.st_mtime;
 	sprintf (dep, "%s/lib/modules/%s", base_dir, force_ver);
 	return xftw (dep, check_update);
 }
@@ -419,6 +425,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 	char depfile_tmp[PATH_MAX];
 	char pcimapfile_tmp[PATH_MAX];
 	char isapnpmapfile_tmp[PATH_MAX];
+	char usbmapfile_tmp[PATH_MAX];
 	char **glb;
 	char old_name[] = "/etc/conf.modules";
 	int conf_file_specified = 0;
@@ -575,6 +582,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 	depfile_tmp[0] = '\0';
 	pcimapfile_tmp[0] = '\0';
 	isapnpmapfile_tmp[0] = '\0';
+	usbmapfile_tmp[0] = '\0';
 
 	/* Only read the default entries on the first file */
 	if (depth == 0) {
@@ -712,6 +720,11 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			strncpy(isapnpmapfile_tmp, envpath, sizeof(isapnpmapfile_tmp));
 		}
 
+		/* USBMAPPATH is intended for testing only */
+		if ((envpath = getenv("USBMAPPATH")) != NULL) {
+			strncpy(usbmapfile_tmp, envpath, sizeof(usbmapfile_tmp));
+		}
+
 	}	/* End of depth == 0 */
 
 	if (conf_file ||
@@ -1207,6 +1220,24 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			one_err = 0;
 		}
 
+		/*
+		 * usbmapfile=
+		 */
+		else if (assgn && (strcmp(parm, "usbmapfile") == 0) &&
+			 (usbmapfile_tmp[0] == '\0')) {
+			/*
+			 * The default value for the usbmapfile parameter is:
+			 *
+			 * usbmapfile=/lib/modules/`uname -r`/modules.usbmap
+			 *
+			 * If the config file exist but doesn't have a usbmapfile
+			 * specification, the default is used since modprobe
+			 * can't work without one.
+			 */
+			strcpy(usbmapfile_tmp, arg);
+			one_err = 0;
+		}
+
 		/*
 		 * keep
 		 */
@@ -1415,6 +1446,36 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			isapnpmapfile = g.pathv[0];
 	}
 
+	/*
+	 * Make sure we have a definition of "usbmapfile"
+	 */
+	if (usbmapfile_tmp[0] == '\0') {
+		/*
+		 * Specification: config file / no usbmapfile parameter
+		 * The default value for the usbmapfile parameter is:
+		 *
+		 * usbmapfile=/lib/modules/`uname -r`/modules.usbmap
+		 *
+		 * If the config file exists but lack a usbmapfile
+		 * specification, the default value is used since
+		 * the system can't work without one.
+		 */
+		sprintf(usbmapfile_tmp, "%s/lib/modules/%s/modules.usbmap",
+			base_dir, version);
+		usbmapfile = xstrdup(usbmapfile_tmp);
+	} else { /* usbmapfile defined in modules.conf */
+		/*
+		 * If we have a usbmapfile definition in the configuration file
+		 * we must resolve any shell meta-chars in its value.
+		 */
+		if (meta_expand(usbmapfile_tmp, &g, base_dir, version))
+			ret = -1;
+		else if (!g.pathv || g.pathv[0] == NULL)
+			ret = -1;
+		else
+			usbmapfile = g.pathv[0];
+	}
+
 	return ret;
 }
 
